#!/bin/awk -f
# postprocess Factorio's console/log output
BEGIN{OFS=FS="|"; smoothe=18000} # smoothing on prod rates, ticks (18k=5m)
/^Running/{next}
/^ *([][0-9.]* )*Info (Blueprint|AppManager|ChildProcessAgent|AsyncScenarioSaver)/{next}
/^Performed/ {
	# Performed 216000 updates in 80704.000 ms
	print
	patsplit($0,words,"[^ ]+")
	for (i in smoothprod) {
		if ( smoothmax[i]==0 || (smoothmax[i]-smoothmin[i])/smoothmax[i] < 0.005 ) {
			printf "%s\t%.6g\t  per cpu ms\n",i,smoothprod[i]*words[2]/(words[5]-overhead)
			printf "%s\t%.6g\t  per minute\n",i,smoothprod[i]*3600
			}
		else {
			smax=smoothmax[i]*words[2]/words[5]
			smin=smoothmin[i]*words[2]/words[5]
			printf "%s\t%.6g-%.6g\t  per cpu ms\n",i,smin,smax
			pmax=smoothmax[i]*3600
			pmin=smoothmin[i]*3600
			printf "%s\t%.6g-%.6g\t  per minute\n",i,pmin,pmax
			}
		}
	Performed=1
	next
	}

END{ if (!Performed) rawprod() }
$0~/^[ 0-9.]*Loading map/ { rawprod() }
function rawprod() {
	for (i in smoothprod) {
		if ( smoothmax[i]==0 || (smoothmax[i]-smoothmin[i])/smoothmax[i] < 0.005 ) {
			printf "%s\t%.6g\t  per minute\n",i,smoothprod[i]*3600
			}
		else {
			pmax=smoothmax[i]*3600
			pmin=smoothmin[i]*3600
			printf "%s\t%.6g-%.6g\t  per minute\n",i,pmin,pmax
			}
		}
	split("",smoothprod)
	split("",smoothmax)
	split("",smoothmin)
	split("",lasttick)
	split("",lastprod)
	split("",basetick)
	split("",baseprod)
	}

NF<14	{if(!quiet) print; next}
$1!~/:/ {if(!quiet) print
	# header line, get field numbers
	for (i=0;i++<NF;) field[$i]=i

	act=field["act"];      count=field["#"];    item=field["item"];
	tick=field["tick"];    wait=field["wait"];  prod=field["tprod"];
	sname=field["sname"];  sx=field["sx"];      sy=field["sy"];
	tname=field["tname"];  tid=field["tid"];
	del=field["del"];      pu=field["pu"];      keep=field["keep"];

	next
	}
!($item in basetick) {
	basetick[$item] = $tick; baseprod[$item] = $prod
	lasttick[$item] = $tick; lastprod[$item] = $prod
	}

$prod>0 {
	interval=$tick-lasttick[$item]
	baseline=$tick-basetick[$item]
	if ( interval*baseline == 0 ) {if(!quiet)print;next}
	newprod=$prod-lastprod[$item]
	newrate=newprod/interval
	lastprod[$item]=$prod
	lasttick[$item]=$tick
	if (!($item in smoothprod)) { smoothprod[$item]=newrate }
	else {
		if (baseline > smoothe) baseline = smoothe
		smoothfac = (interval<baseline? interval/baseline : 1)
		s=smoothprod[$item]*(1-smoothfac)+newrate*smoothfac
		smoothprod[$item]=s
		if (baseline>=smoothe) {
			if (!($item in smoothmin) || smoothmin[$item]>s)
				smoothmin[$item] = s
			if (!($item in smoothmax) || smoothmax[$item]<s)
				smoothmax[$item] = s
			}
		}
#	if ( NF==14 ) {
#		$++NF=newrate*3600
#		$++NF=smoothprod[$item]*3600
#		$++NF=($prod-baseprod[$item])/($tick-basetick[$item])*3600
#		}
	}
!quiet

BEGIN{	act=1;     count=2;  item=3;   #  default field positions for c&p
	tick=4;    wait=5;   tprod=6;
	sname=7;   sx=8;     sy=9;
	tname=10;  tid=11;
	del=12;    pu=13;    keep=14
	}
