-- handy-dandies for console use
red=defines.wire_type.red
green=defines.wire_type.green
mfd='marked_for_deconstruction'
pda='player_deconstructed_area'
nul=setmetatable({},
    {__newindex=function()end,__call=function() return nil end})
empty=setmetatable({},
    {__newindex=function()end,__call=function(self) return self end})
-- handy table meta, htm(t) allows t:insert(1) and for k,v in t
htm = setmetatable(
    {__index=table,__call=function(s,_,k) return next(s,k) end},
    {__call=function(s,t) return setmetatable(t or {},s) end})

psd={comment=false,nocode=true}
function psl(p,k) return print(serpent.line(p,k or psd)) end
function psb(p,k) return print(serpent.block(p,k or psd)) end
function gpsl(p,k) return game.print(serpent.line(p,k or psd)) end

-- /c locos=fef'locomotive'
-- /c for _,l in locos do l.insert'rocket-fuel' end
function fef(t,e)
    if not t then t={} end
    if type(t)=='string' then t={name=t} end
    if type(t[1])=='string' then t.name=t[1] t[1]=nil end
    if not t.name and not t.type and e then t.name=e.name end
    if not t.hop then
	return htm((e and e.surface or game.surfaces[1]) .
					find_entities_filtered(t))
	end
    end

-- row-major
function topmost(a,b) local ap, bp= a.position or a, b.position or b
			return ap.y < bp.y
                           or ( ap.y == bp.y and ap.x < bp.x ) end
-- column-major
function leftmost(a,b) local ap, bp= a.position or a, b.position or b
			return ap.x < bp.x
                           or ( ap.x == bp.x and ap.y < bp.y ) end
-- nearest
function nearest(e)
    local pos=e.position or e
    return function(a,b)
	local ap,bp = a.position or a, b.position or b
	local axdiff,aydiff = ap.x-pos.x, ap.y-pos.y
	local bxdiff,bydiff = bp.x-pos.x, bp.y-pos.y
	return (axdiff*axdiff+aydiff*aydiff) < (bxdiff*bxdiff+bydiff*bydiff)
	end
    end

default_from_template={
    all={position=1,direction=1,force=1,surface=1}
    ,['underground-belt']={belt_to_ground_type='type'}
}
function ce(t,e)
    local pidx
    if type(t)=='string' then t={name=t} end
    if type(t[1])=='string' then t.name=t[1] t[1]=t[#t] t[#t]=nil  end
    if type(t[1])=='table' then t.position=t[1] t[1]=t[#t] t[#t]=nil  end
    if e and type(e)=='table' then
	if type(e.__self)=='userdata' then
	    -- load up defaults from entity template
	    if not t.position then
		t.fast_replace=true
		t.spill=false
		end
	    for k,_ in next,default_from_template.all do
		if not t[k] then t[k]=e[k] end
		end
	    for k,v in next,(default_from_template[e.prototype.type] or nul) do
		if not t[k] then t[v]=e[k] end
		end
	else
	    -- not sure wtd here, event? get player index (for force and surface) from event
	    pidx = e.player_index
	    end
	end
    local p=pidx and game.players[pidx] or game.player
    t.force = t.force or p and p.force or game.forces.player
    return (p and p.surface or game.surfaces[1]).create_entity(t)
    end

function tpto(pos)
    local p = game.player
    back=p.position
    p.teleport(p.surface.find_non_colliding_position(
	    'iron-chest', pos and pos.type and pos.position or pos, 5, 0.25))
    end

local foundsigs={}
function sig(id,c)
    -- return a spec for the given signal id and (optional) count. The
    -- signal type is found in prototypes, omitted count returns just
    -- the id, so
    --     sig'signal-black' → {name='signal-black',type='virtual'}
    -- and   sig('coal',100} → {count=100,signal=sig'coal'}  (further expanded).
    -- for extra fun, the `signal-` prefix is made optional, so
    --     sig'W' → {name='signal-W',type='virtual'}

    if c then return {count=c,signal=sig(id)} end
    if not foundsigs[id] then
	foundsigs[id] = game.virtual_signal_prototypes[id] and	{name=id,type='virtual'}
		     or game.item_prototypes[id] and  {name=id,type='item'}
		     or game.fluid_prototypes[id] and  {name=id,type='fluid'}
		     or game.virtual_signal_prototypes['signal-'..id]
					    and  {name='signal-'..id,type='virtual'}
		     or {name='**unknown signal id "'..id..'"'}
	local _, _2, short = foundsigs[id].name:find'^signal%-(.*)'
	if _ then
	    foundsigs[foundsigs[id].name] = foundsigs[id] -- with signal-
	    foundsigs[short]              = foundsigs[id] -- and without it
	    end
        end
    return foundsigs[id]
    end

function ccpar(...)
    -- build constant-combinator parameters, e.g.
    -- entity.get_control_behavior().parameters = ccpar(100,'coal')
    -- entity.get_control_behavior().parameters
    --                           = ccpar({count=0},'O','H','H','I')
    -- pars = ccpar'black'
    -- plist = pars.parameters
    -- for k,v in cccblist do plist[1].count=k v.parameters=pars end
    local plist={}
    local count,index,init=1,1,{}
    for k,v in next,table.pack(...) do
	if type(v)=='number' then
	    count=v
	elseif type(v)=='table' then
	    init=v count=init.count or 1 index=init.index or 1
	else
	    plist[#plist+1] = {index=index,count=count,signal=sig(v)}
	    index=index+1
	    count=init.count or 1
	    end
	end
    return {parameters=plist}
    end

local weakkeys={__mode='k'}
local function floorall(t,seen)
    for k,v in next,t do
	if type(v)=='number' then t[k]=v-v%1
	elseif type(v)=='table' then
		if not seen then seen = setmetatable({},weakkeys) end
		if not seen[v] then
		    seen[v]=true
		    floorall(v,seen)
		    end
	    end
	end
    end
local all = setmetatable({},{
	__index=function(s,k) return s.value end
    ,	__call=function(s,k) s.value=k end
    })
function tilearea(tile,area)
    if not area then area=iodarea end
    floorall(area)
    local tiles=htm{}
    for x=(area.left_top or all(area[1][1])).x,
	  (area.right_bottom or all(area[2][1])).x do
	for y=(area.left_top or all(area[1][2])).y,
	      (area.right_bottom or all(area[2][2])).y do
	    tiles:insert{name=tile,position={x,y}}
	    end
	game.surfaces[1].set_tiles(tiles)
	end
    end

-- Set up e.g. B(e,'modules') to look up di[e.type..'_modules']
-- and return the corresponding inventory.
bkey = setmetatable({},{
    __index=function(s,k) s[k]={} return s[k] end
})
for k,v in next,defines.inventory do
    local _,_,t,b = k:find'(.*)_(.*)'
    if b then bkey[t:gsub('_','-')][b]=v else bkey[k]=v end
    end
function B(e,n)
    return e and e.valid and e.get_inventory(
	type(n)=='number' and n or bkey[e.type][n]
    ) or nil
    end
